@@ -787,14 +787,11 @@ class ApplicationSession(BaseSession):
                             else:
                                 kw = msg.kwargs or dict()
                                 args = msg.args or tuple()
-                                try:
-                                    # XXX what if on_progress returns a Deferred/Future?
-                                    call_request.options.on_progress(*args, **kw)
-                                except Exception:
-                                    try:
-                                        self.onUserError(txaio.create_failure(), "While firing on_progress")
-                                    except:
-                                        pass
+
+                                def _error(fail):
+                                    self.onUserError(fail, "While firing on_progress")
+                                prog_d = txaio.as_future(call_request.options.on_progress, *args, **kw)
+                                txaio.add_callbacks(prog_d, None, _error)
 
                     else:
                         # process final call result
@@ -1109,7 +1106,8 @@ class ApplicationSession(BaseSession):
                     on_reply = self._unregister_reqs.pop(msg.request).on_reply
 
                 if on_reply:
-                    txaio.reject(on_reply, self._exception_from_message(msg))
+                    if not txaio.is_called(on_reply):
+                        txaio.reject(on_reply, self._exception_from_message(msg))
                 else:
                     raise ProtocolError("WampAppSession.onMessage(): ERROR received for non-pending request_type {0} and request ID {1}".format(msg.request_type, msg.request))
 
